@@ -289,7 +289,7 @@ int parse_options(int argc, char **argv, aacenc_param_ex_t *params)
     params->afterburner = 1;
 
     aacenc_getmainargs(&argc, &argv);
-    while ((ch = getopt_long(argc, argv, "hp:b:m:w:a:Ls:f:CP:G:Io:SR",
+    while ((ch = getopt_long(argc, argv, "hp:b:m:w:a:L:s:f:CP:G:Io:SR",
                              long_options, 0)) != EOF) {
         switch (ch) {
         case 'h':
@@ -503,6 +503,11 @@ int write_sample(FILE *ofp, m4af_ctx_t *m4af, aacenc_frame_t *frame)
     return 0;
 }
 
+static int do_smart_padding(int profile)
+{
+    return profile == 2 || profile == 5 || profile == 29;
+}
+
 static
 int encode(aacenc_param_ex_t *params, pcm_reader_t *reader,
            HANDLE_AACENCODER encoder, uint32_t frame_length, 
@@ -517,15 +522,12 @@ int encode(aacenc_param_ex_t *params, pcm_reader_t *reader,
     int frames_written = 0, encoded = 0;
     aacenc_progress_t progress = { 0 };
     const pcm_sample_description_t *fmt = pcm_get_format(reader);
+    const int is_padding = do_smart_padding(params->profile);
 
     ibuf = malloc(frame_length * fmt->bytes_per_frame);
     aacenc_progress_init(&progress, pcm_get_length(reader), fmt->sample_rate);
 
     for (;;) {
-        /*
-         * Since we delay the write, we cannot just exit loop when interrupted.
-         * Instead, we regard it as EOF.
-         */
         if (g_interrupted)
             nread = 0;
         if (nread > 0) {
@@ -548,7 +550,7 @@ int encode(aacenc_param_ex_t *params, pcm_reader_t *reader,
 
             remaining -= consumed;
             ip += consumed * fmt->channels_per_frame;
-            flip ^= 1;
+            if (is_padding) {
             /*
              * As we pad 1 frame at beginning and ending by our extrapolator,
              * we want to drop them.
@@ -557,25 +559,26 @@ int encode(aacenc_param_ex_t *params, pcm_reader_t *reader,
              * Since sbr_header is included in the first frame (in case of
              * SBR), we cannot discard first frame. So we pick second instead.
              */
-            ++encoded;
-            if (encoded == 1 || encoded == 3)
-                continue;
-
+                flip ^= 1;
+                ++encoded;
+                if (encoded == 1 || encoded == 3)
+                    continue;
+            }
             if (write_sample(params->output_fp, m4af, &obuf[flip]) < 0)
                 goto END;
             ++frames_written;
         } while (remaining > 0);
-        /*
-         * When interrupted, we haven't pulled out last extrapolated frames
-         * from the reader. Therefore, we have to write the final outcome.
-         */
-        if (g_interrupted) {
-            if (write_sample(params->output_fp, m4af, &obp[flip^1]) < 0)
-                goto END;
-            ++frames_written;
-        }
     }
 DONE:
+    /*
+     * When interrupted, we haven't pulled out last extrapolated frames
+     * from the reader. Therefore, we have to write the final outcome.
+     */
+    if (g_interrupted) {
+        if (write_sample(params->output_fp, m4af, &obp[flip^1]) < 0)
+            goto END;
+        ++frames_written;
+    }
     if (!params->silent)
         aacenc_progress_finish(&progress, pcm_get_position(reader));
     rc = frames_written;
@@ -755,8 +758,10 @@ pcm_reader_t *open_input(aacenc_param_ex_t *params)
     reader = pcm_open_native_converter(reader);
     if (reader && PCM_IS_FLOAT(pcm_get_format(reader)))
         reader = limiter_open(reader);
-    if (reader && (reader = pcm_open_sint16_converter(reader)) != 0)
-        reader = extrapolater_open(reader);
+    if (reader && (reader = pcm_open_sint16_converter(reader)) != 0) {
+        if (do_smart_padding(params->profile))
+            reader = extrapolater_open(reader);
+    }
     return reader;
 FAIL:
     return 0;
@@ -846,29 +851,22 @@ int main(int argc, char **argv)
         m4af_set_priming_mode(m4af, params.gapless_mode + 1);
         m4af_begin_write(m4af);
     }
-    if (scale_shift && (aacinfo.encoderDelay & 1)) {
-        /*
-         * Since odd delay cannot be exactly expressed in downsampled scale,
-         * we push one zero frame to the encoder here, to make delay even
-         */
-        int16_t zero[8] = { 0 };
-        aacenc_frame_t frame = { 0 };
-        aac_encode_frame(encoder, sample_format, zero, 1, &frame);
-        free(frame.data);
-    }
     frame_count = encode(&params, reader, encoder, aacinfo.frameLength, m4af);
     if (frame_count < 0)
         goto END;
     if (m4af) {
-        uint32_t delay = aacinfo.encoderDelay;
         uint32_t padding;
+#if AACENCODER_LIB_VL0 < 4
+        uint32_t delay = aacinfo.encoderDelay;
+        if (sbr_mode && params.profile != AOT_ER_AAC_ELD
+            && !params.include_sbr_delay)
+            delay -= 481 << scale_shift;
+#else
+        uint32_t delay = params.include_sbr_delay ? aacinfo.nDelay
+                                                  : aacinfo.nDelayCore;
+#endif
         int64_t frames_read = pcm_get_position(reader);
 
-        if (sbr_mode && params.profile != AOT_ER_AAC_ELD &&
-            !params.include_sbr_delay)
-            delay -= 481 << scale_shift;
-        if (scale_shift && (delay & 1))
-            ++delay;
         padding = frame_count * aacinfo.frameLength - frames_read - delay;
         m4af_set_priming(m4af, 0, delay >> scale_shift, padding >> scale_shift);
         if (finalize_m4a(m4af, &params, encoder) < 0)
